@@ -7,8 +7,9 @@
 #
 # See HighLine for documentation.
 #
-# This is Free Software.  See LICENSE and COPYING for details
+# This is Free Software.  See LICENSE and COPYING for details.
 
+require "highline/system_extensions"
 require "highline/question"
 require "highline/menu"
 require "erb"
@@ -27,7 +28,7 @@ require "abbrev"
 #
 class HighLine
   # The version of the installed library.
-  VERSION = "1.2.0".freeze
+  VERSION = "1.2.2".freeze
   
   # An internal HighLine error.  User code does not need to trap this.
   class QuestionError < StandardError
@@ -111,8 +112,9 @@ class HighLine
                   wrap_at = nil, page_at = nil )
     @input   = input
     @output  = output
-    @wrap_at = wrap_at
-    @page_at = page_at
+    
+    self.wrap_at = wrap_at
+    self.page_at = page_at
     
     @question = nil
     @answer   = nil
@@ -124,16 +126,12 @@ class HighLine
     @key      = nil
   end
   
-  #
-  # Set to an integer value to cause HighLine to wrap output lines at the
-  # indicated character limit.  When +nil+, the default, no wrapping occurs.
-  #
-  attr_accessor :wrap_at
-  #
-  # Set to an integer value to cause HighLine to page output lines over the
-  # indicated line limit.  When +nil+, the default, no paging occurs.
-  #
-  attr_accessor :page_at
+  include HighLine::SystemExtensions
+  
+  # The current column setting for wrapping output.
+  attr_reader :wrap_at
+  # The current row setting for paging output.
+  attr_reader :page_at
   
   #
   # A shortcut to HighLine.ask() a question that only accepts "yes" or "no"
@@ -414,6 +412,48 @@ class HighLine
     end
   end
   
+  #
+  # Set to an integer value to cause HighLine to wrap output lines at the
+  # indicated character limit.  When +nil+, the default, no wrapping occurs.  If
+  # set to <tt>:auto</tt>, HighLine will attempt to determing the columns
+  # available for the <tt>@output</tt> or use a sensible default.
+  #
+  def wrap_at=( setting )
+    @wrap_at = setting == :auto ? output_cols : setting
+  end
+  
+  #
+  # Set to an integer value to cause HighLine to page output lines over the
+  # indicated line limit.  When +nil+, the default, no paging occurs.  If
+  # set to <tt>:auto</tt>, HighLine will attempt to determing the rows available
+  # for the <tt>@output</tt> or use a sensible default.
+  #
+  def page_at=( setting )
+    @page_at = setting == :auto ? output_rows : setting
+  end
+  
+  # 
+  # Returns the number of columns for the console, or a default it they cannot
+  # be determined.
+  # 
+  def output_cols
+    return 80 unless @output.tty?
+    terminal_size.first
+  rescue
+    return 80
+  end
+  
+  # 
+  # Returns the number of rows for the console, or a default if they cannot be
+  # determined.
+  # 
+  def output_rows
+    return 24 unless @output.tty?
+    terminal_size.last
+  rescue
+    return 24
+  end
+  
   private
   
   #
@@ -475,69 +515,6 @@ class HighLine
     
     @answers
   end
-  
-  #
-  # This section builds a character reading function to suit the proper
-  # platform we're running on.  Be warned:  Here be dragons!
-  #
-  begin
-    require "Win32API"       # See if we're on Windows.
-
-    CHARACTER_MODE = "Win32API"    # For Debugging purposes only.
-
-    #
-    # Windows savvy getc().
-    # 
-    # *WARNING*:  This method ignores <tt>@input</tt> and reads one
-    # character from +STDIN+!
-    # 
-    def get_character
-      Win32API.new("crtdll", "_getch", [ ], "L").Call
-    end
-  rescue LoadError             # If we're not on Windows try...
-    begin
-      require "termios"    # Unix, first choice.
-    
-      CHARACTER_MODE = "termios"    # For Debugging purposes only.
-
-      #
-      # Unix savvy getc().  (First choice.)
-      # 
-      # *WARNING*:  This method requires the "termios" library!
-      # 
-      def get_character
-        old_settings = Termios.getattr(@input)
-
-        new_settings         =  old_settings.dup
-        new_settings.c_lflag &= ~(Termios::ECHO | Termios::ICANON)
-    
-        begin
-          Termios.setattr(@input, Termios::TCSANOW, new_settings)
-          @input.getc
-        ensure
-          Termios.setattr(@input, Termios::TCSANOW, old_settings)
-        end
-      end
-    rescue LoadError         # If our first choice fails, default.
-      CHARACTER_MODE = "stty"    # For Debugging purposes only.
-
-      #
-      # Unix savvy getc().  (Second choice.)
-      # 
-      # *WARNING*:  This method requires the external "stty" program!
-      # 
-      def get_character
-        state = `stty -g`
-        
-        begin
-          system "stty raw -echo cbreak"
-          @input.getc
-        ensure
-          system "stty #{state}"
-        end
-      end
-    end
-  end
 
   #
   # Read a line of input from the input stream and process whitespace as
@@ -595,22 +572,29 @@ class HighLine
       if @question.echo == true and @question.limit.nil?
         get_line
       else
+        raw_no_echo_mode if stty = CHARACTER_MODE == "stty"
+        
         line = ""
-        while character = get_character
-          line << character.chr
-          # looking for carriage return (decimal 13) or
-          # newline (decimal 10) in raw input
-          break if character == 13 or character == 10 or
-                   (@question.limit and line.size == @question.limit)
-          @output.print(@question.echo) if @question.echo != false
+        begin
+          while character = (stty ? @input.getc : get_character(@input))
+            line << character.chr
+            # looking for carriage return (decimal 13) or
+            # newline (decimal 10) in raw input
+            break if character == 13 or character == 10 or
+                     (@question.limit and line.size == @question.limit)
+            @output.print(@question.echo) if @question.echo != false
+          end
+          say("\n")
+        ensure
+          restore_mode if stty
         end
-        say("\n")
+        
         @question.change_case(@question.remove_whitespace(line))
       end
     elsif @question.character == :getc
       @question.change_case(@input.getc.chr)
     else
-      response = get_character.chr
+      response = get_character(@input).chr
       echo = if @question.echo == true
         response
       elsif @question.echo != false
@@ -636,12 +620,23 @@ class HighLine
     while lines.size > @page_at
       @output.puts lines.slice!(0...@page_at).join
       @output.puts
-      ask("-- press enter/return to continue -- ")
-      @output.puts
+      # Return last line if user wants to abort paging
+      return (["...\n"] + lines.slice(-2,1)).join unless continue_paging?
     end
     return lines.join
   end
-  
+ 
+  # 
+  # Ask user if they wish to continue paging output. Allows them to type "q" to
+  # cancel the paging process.
+  # 
+  def continue_paging?
+    command = HighLine.new(@input, @output).ask(
+      "-- press enter/return to continue or q to stop -- "
+    ) { |q| q.character = true }
+    command !~ /\A[qQ]\Z/  # Only continue paging if Q was not hit.
+  end
+    
   #
   # Wrap a sequence of _lines_ at _wrap_at_ characters per line.  Existing
   # newlines will not be affected by this process, but additional newlines
@@ -666,6 +661,10 @@ class HighLine
     return wrapped.join
   end
   
+  # 
+  # Returns the length of the passed +string_with_escapes+, minus and color
+  # sequence escapes.
+  # 
   def actual_length( string_with_escapes )
     string_with_escapes.gsub(/\e\[\d{1,2}m/, "").length
   end
