@@ -12,6 +12,15 @@ require "test/unit"
 require "highline"
 require "stringio"
 
+if HighLine::CHARACTER_MODE == "Win32API"
+  class HighLine
+    # Override Windows' character reading so it's not tied to STDIN.
+    def get_character( input = STDIN )
+      input.getc
+    end
+  end
+end
+
 class TestHighLine < Test::Unit::TestCase
   def setup
     @input    = StringIO.new
@@ -374,9 +383,7 @@ class TestHighLine < Test::Unit::TestCase
   end
   
   def test_mode
-    # *WARNING*:  These tests wil only complete is "stty" mode!
-    assert_equal( "stty", HighLine::CHARACTER_MODE,
-                  "Tests require \"stty\" mode." )
+    assert(%w[Win32API termios stty].include?(HighLine::CHARACTER_MODE))
   end
   
   class NameClass
@@ -443,9 +450,9 @@ class TestHighLine < Test::Unit::TestCase
 
     @terminal.say((1..50).map { |n| "This is line #{n}.\n"}.join)
     assert_equal( (1..22).map { |n| "This is line #{n}.\n"}.join +
-                  "\n-- press enter/return to continue -- \n" +
+                  "\n-- press enter/return to continue or q to stop -- \n\n" +
                   (23..44).map { |n| "This is line #{n}.\n"}.join +
-                  "\n-- press enter/return to continue -- \n" +
+                  "\n-- press enter/return to continue or q to stop -- \n\n" +
                   (45..50).map { |n| "This is line #{n}.\n"}.join,
                   @output.string )
   end
